@@ -66,7 +66,7 @@
 
 #define FL_PARENT_FOLDER_ELEMENT "<parent-folder/>" EOL_CHARS
 
-#define FL_FILE_ELEMENT "<file name=\"%s\" size=\"%lu\"" \
+#define FL_FILE_ELEMENT "<file name=\"%s\" size=\"%" PRIu64 "\"" \
 			" %s accessed=\"%s\" " \
 			"modified=\"%s\" created=\"%s\"/>" EOL_CHARS
 
@@ -121,8 +121,9 @@ static char *file_stat_line(char *filename, struct stat *fstat,
 			ret = g_strdup_printf(FL_FOLDER_ELEMENT, escaped, perm,
 							atime, mtime, ctime);
 	} else if (S_ISREG(fstat->st_mode))
-		ret = g_strdup_printf(FL_FILE_ELEMENT, escaped, fstat->st_size,
-						perm, atime, mtime, ctime);
+		ret = g_strdup_printf(FL_FILE_ELEMENT, escaped,
+					(uint64_t) fstat->st_size,
+					perm, atime, mtime, ctime);
 
 	g_free(escaped);
 
@@ -209,7 +210,7 @@ static int filesystem_close(void *object)
 }
 
 static ssize_t filesystem_read(void *object, void *buf, size_t count,
-								uint8_t *hi)
+					uint8_t *hi, unsigned int *flags)
 {
 	ssize_t ret;
 
@@ -217,6 +218,9 @@ static ssize_t filesystem_read(void *object, void *buf, size_t count,
 	if (ret < 0)
 		return -errno;
 
+	if (flags)
+		*flags = 0;
+
 	*hi = OBEX_HDR_BODY;
 
 	return ret;
@@ -474,7 +478,7 @@ static void *pcsuite_open(const char *name, int oflag, mode_t mode,
 	return append_listing(object, name, TRUE, size, err);
 }
 
-int string_free(void *object)
+static int string_free(void *object)
 {
 	GString *string = object;
 
@@ -498,17 +502,24 @@ ssize_t string_read(void *object, void *buf, size_t count)
 	return len;
 }
 
-static ssize_t folder_read(void *object, void *buf, size_t count, uint8_t *hi)
+static ssize_t folder_read(void *object, void *buf, size_t count,
+					uint8_t *hi, unsigned int *flags)
 {
+	if (flags)
+		*flags = 0;
+
 	*hi = OBEX_HDR_BODY;
 	return string_read(object, buf, count);
 }
 
 static ssize_t capability_read(void *object, void *buf, size_t count,
-								uint8_t *hi)
+					uint8_t *hi, unsigned int *flags)
 {
 	struct capability_object *obj = object;
 
+	if (flags)
+		*flags = 0;
+
 	*hi = OBEX_HDR_BODY;
 
 	if (obj->buffer)
