@@ -109,7 +109,12 @@
 "}) "									\
 "nco:birthDate(?_contact) "						\
 "nco:nickname(?_contact) "						\
-"nco:url(?_contact) "							\
+"(SELECT GROUP_CONCAT(fn:concat( "					\
+	"?url_val, \"\31\", rdfs:label(?_role) "			\
+	"), \"\30\") "							\
+	"WHERE {"							\
+		"?_role nco:url ?url_val . "				\
+"})"									\
 "nie:url(nco:photo(?_contact)) "					\
 "nco:role(?_role) "							\
 "nco:contactUID(?_contact) "						\
@@ -152,8 +157,12 @@
 "(SELECT fn:concat(rdf:type(?role_number),"				\
 	"\"\31\", nco:phoneNumber(?role_number))"			\
 	"WHERE {"							\
-	"?_role nco:hasPhoneNumber ?role_number "			\
-	"FILTER (?role_number = ?_number)"				\
+	"{"								\
+	"	?_role nco:hasPhoneNumber ?role_number "		\
+	"	FILTER (?role_number = ?_number)"			\
+	"} UNION { "							\
+		"?_unb_contact nco:hasPhoneNumber ?role_number . "	\
+	"}"								\
 "} GROUP BY nco:phoneNumber(?role_number) ) "				\
 	"nco:fullname(?_contact) "					\
 	"nco:nameFamily(?_contact) "					\
@@ -177,7 +186,14 @@
 	"}) "								\
 	"nco:birthDate(?_contact) "					\
 	"nco:nickname(?_contact) "					\
-	"nco:url(?_contact) "						\
+"(SELECT GROUP_CONCAT(fn:concat( "					\
+	"?url_value, \"\31\", ?aff_type "				\
+	"), \"\30\") "							\
+	"WHERE {"							\
+		"?_contact nco:hasAffiliation ?c_role . "		\
+		"?c_role nco:url ?url_value . "				\
+		"?c_role rdfs:label ?aff_type . "			\
+"})"									\
 	"nie:url(nco:photo(?_contact)) "				\
 	"nco:role(?_role) "						\
 	"nco:contactUID(?_contact) "					\
@@ -278,8 +294,12 @@
 "(SELECT fn:concat(rdf:type(?role_number),"				\
 	"\"\31\", nco:phoneNumber(?role_number))"			\
 	"WHERE {"							\
-	"?_role nco:hasPhoneNumber ?role_number "			\
-	"FILTER (?role_number = ?_number)"				\
+	"{"								\
+	"	?_role nco:hasPhoneNumber ?role_number "		\
+	"	FILTER (?role_number = ?_number)"			\
+	"} UNION { "							\
+		"?_unb_contact nco:hasPhoneNumber ?role_number . "	\
+	"}"								\
 "} GROUP BY nco:phoneNumber(?role_number) ) "				\
 	"nco:fullname(?_contact) "					\
 	"nco:nameFamily(?_contact) "					\
@@ -303,7 +323,14 @@
 	"}) "								\
 	"nco:birthDate(?_contact) "					\
 	"nco:nickname(?_contact) "					\
-	"nco:url(?_contact) "						\
+"(SELECT GROUP_CONCAT(fn:concat( "					\
+	"?url_value, \"\31\", ?aff_type "				\
+	"), \"\30\") "							\
+	"WHERE {"							\
+		"?_contact nco:hasAffiliation ?c_role . "		\
+		"?c_role nco:url ?url_value . "				\
+		"?c_role rdfs:label ?aff_type . "			\
+"})"									\
 	"nie:url(nco:photo(?_contact)) "				\
 	"nco:role(?_role) "						\
 	"nco:contactUID(?_contact) "					\
@@ -403,8 +430,12 @@
 "(SELECT fn:concat(rdf:type(?role_number),"				\
 	"\"\31\", nco:phoneNumber(?role_number))"			\
 	"WHERE {"							\
-	"?_role nco:hasPhoneNumber ?role_number "			\
-	"FILTER (?role_number = ?_number)"				\
+	"{"								\
+	"	?_role nco:hasPhoneNumber ?role_number "		\
+	"	FILTER (?role_number = ?_number)"			\
+	"} UNION { "							\
+		"?_unb_contact nco:hasPhoneNumber ?role_number . "	\
+	"}"								\
 "} GROUP BY nco:phoneNumber(?role_number) ) "				\
 	"nco:fullname(?_contact) "					\
 	"nco:nameFamily(?_contact) "					\
@@ -428,7 +459,14 @@
 	"}) "								\
 	"nco:birthDate(?_contact) "					\
 	"nco:nickname(?_contact) "					\
-	"nco:url(?_contact) "						\
+"(SELECT GROUP_CONCAT(fn:concat( "					\
+	"?url_value, \"\31\", ?aff_type "				\
+	"), \"\30\") "							\
+	"WHERE {"							\
+		"?_contact nco:hasAffiliation ?c_role . "		\
+		"?c_role nco:url ?url_value . "				\
+		"?c_role rdfs:label ?aff_type . "			\
+"})"									\
 	"nie:url(nco:photo(?_contact)) "				\
 	"nco:role(?_role) "						\
 	"nco:contactUID(?_contact) "					\
@@ -522,8 +560,12 @@
 "(SELECT fn:concat(rdf:type(?role_number),"				\
 	"\"\31\", nco:phoneNumber(?role_number))"			\
 	"WHERE {"							\
-	"?_role nco:hasPhoneNumber ?role_number "			\
-	"FILTER (?role_number = ?_number)"				\
+	"{"								\
+	"	?_role nco:hasPhoneNumber ?role_number "		\
+	"	FILTER (?role_number = ?_number)"			\
+	"} UNION { "							\
+		"?_unb_contact nco:hasPhoneNumber ?role_number . "	\
+	"}"								\
 "} GROUP BY nco:phoneNumber(?role_number) ) "				\
 	"nco:fullname(?_contact) "					\
 	"nco:nameFamily(?_contact) "					\
@@ -547,7 +589,14 @@
 	"}) "								\
 	"nco:birthDate(?_contact) "					\
 	"nco:nickname(?_contact) "					\
-	"nco:url(?_contact) "						\
+"(SELECT GROUP_CONCAT(fn:concat( "					\
+	"?url_value, \"\31\", ?aff_type "				\
+	"), \"\30\") "							\
+	"WHERE {"							\
+		"?_contact nco:hasAffiliation ?c_role . "		\
+		"?c_role nco:url ?url_value . "				\
+		"?c_role rdfs:label ?aff_type . "			\
+"})"									\
 	"nie:url(nco:photo(?_contact)) "				\
 	"nco:role(?_role) "						\
 	"nco:contactUID(?_contact) "					\
@@ -716,7 +765,12 @@
 "}) "									\
 "nco:birthDate(<%s>) "							\
 "nco:nickname(<%s>) "							\
-"nco:url(<%s>) "							\
+"(SELECT GROUP_CONCAT(fn:concat( "					\
+	"?url_val, \"\31\", rdfs:label(?_role) "			\
+	"), \"\30\") "							\
+	"WHERE {"							\
+		"?_role nco:url ?url_val . "				\
+"})"									\
 "nie:url(nco:photo(<%s>)) "						\
 "nco:role(?_role) "							\
 "nco:contactUID(<%s>) "							\
@@ -741,10 +795,10 @@
 "ORDER BY ?_key tracker:id(<%s>)"
 
 #define CONTACTS_OTHER_QUERY_FROM_URI					\
-	"SELECT \"\" \"\" \"\" \"\" \"\" \"\" \"\" \"\" \"\" \"\" "	\
-	"fn:concat(\"TYPE_OTHER\", \"\31\", nco:phoneNumber(?t)) \"\" "	\
+	"SELECT fn:concat(\"TYPE_OTHER\", \"\31\", nco:phoneNumber(?t))"\
+	"\"\" \"\" \"\" \"\" \"\" \"\" \"\" \"\" "			\
 	"\"\" \"\" \"\" \"\" \"\" \"\" \"\" \"\" \"\" \"\" "		\
-	" \"NOTACALL\" \"false\" \"false\" <%s> "	\
+	" \"NOTACALL\" \"false\" \"false\" <%s> "			\
 	"WHERE { "							\
 		"<%s> a nco:Contact . "					\
 		"OPTIONAL { <%s> nco:hasPhoneNumber ?t . } "		\
@@ -840,6 +894,9 @@
 typedef void (*reply_list_foreach_t) (char **reply, int num_fields,
 							void *user_data);
 
+typedef void (*add_field_t) (struct phonebook_contact *contact,
+						const char *value, int type);
+
 struct pending_reply {
 	reply_list_foreach_t callback;
 	void *user_data;
@@ -1162,17 +1219,17 @@ static struct phonebook_contact *find_contact(GSList *contacts, const char *id)
 	return NULL;
 }
 
-static struct phonebook_number *find_phone(GSList *numbers, const char *phone,
+static struct phonebook_field *find_field(GSList *fields, const char *value,
 								int type)
 {
 	GSList *l;
 
-	for (l = numbers; l; l = l->next) {
-		struct phonebook_number *pb_num = l->data;
+	for (l = fields; l; l = l->next) {
+		struct phonebook_field *field = l->data;
 		/* Returning phonebook number if phone values and type values
 		 * are equal */
-		if (g_strcmp0(pb_num->tel, phone) == 0 && pb_num->type == type)
-			return pb_num;
+		if (g_strcmp0(field->text, value) == 0 && field->type == type)
+			return field;
 	}
 
 	return NULL;
@@ -1181,91 +1238,81 @@ static struct phonebook_number *find_phone(GSList *numbers, const char *phone,
 static void add_phone_number(struct phonebook_contact *contact,
 						const char *phone, int type)
 {
-	struct phonebook_number *number;
+	struct phonebook_field *number;
 
 	if (phone == NULL || strlen(phone) == 0)
 		return;
 
 	/* Not adding number if there is already added with the same value */
-	if (find_phone(contact->numbers, phone, type))
+	if (find_field(contact->numbers, phone, type))
 		return;
 
-	number = g_new0(struct phonebook_number, 1);
-	number->tel = g_strdup(phone);
+	number = g_new0(struct phonebook_field, 1);
+	number->text = g_strdup(phone);
 	number->type = type;
 
 	contact->numbers = g_slist_append(contact->numbers, number);
 }
 
-static struct phonebook_email *find_email(GSList *emails, const char *address,
-								int type)
-{
-	GSList *l;
-
-	for (l = emails; l; l = l->next) {
-		struct phonebook_email *email = l->data;
-		if (g_strcmp0(email->address, address) == 0 &&
-						email->type == type)
-			return email;
-	}
-
-	return NULL;
-}
-
 static void add_email(struct phonebook_contact *contact, const char *address,
 								int type)
 {
-	struct phonebook_email *email;
+	struct phonebook_field *email;
 
 	if (address == NULL || strlen(address) == 0)
 		return;
 
 	/* Not adding email if there is already added with the same value */
-	if (find_email(contact->emails, address, type))
+	if (find_field(contact->emails, address, type))
 		return;
 
-	email = g_new0(struct phonebook_email, 1);
-	email->address = g_strdup(address);
+	email = g_new0(struct phonebook_field, 1);
+	email->text = g_strdup(address);
 	email->type = type;
 
 	contact->emails = g_slist_append(contact->emails, email);
 }
 
-static struct phonebook_address *find_address(GSList *addresses,
-					const char *address, int type)
-{
-	GSList *l;
-
-	for (l = addresses; l; l = l->next) {
-		struct phonebook_address *addr = l->data;
-		if (g_strcmp0(addr->addr, address) == 0 &&
-						addr->type == type)
-			return addr;
-	}
-
-	return NULL;
-}
-
 static void add_address(struct phonebook_contact *contact,
 					const char *address, int type)
 {
-	struct phonebook_address *addr;
+	struct phonebook_field *addr;
 
 	if (address == NULL || address_fields_present(address) == FALSE)
 		return;
 
 	/* Not adding address if there is already added with the same value */
-	if (find_address(contact->addresses, address, type))
+	if (find_field(contact->addresses, address, type))
 		return;
 
-	addr = g_new0(struct phonebook_address, 1);
+	addr = g_new0(struct phonebook_field, 1);
 
-	addr->addr = g_strdup(address);
+	addr->text = g_strdup(address);
 	addr->type = type;
 
 	contact->addresses = g_slist_append(contact->addresses, addr);
 }
 
+static void add_url(struct phonebook_contact *contact, const char *url_val,
+								int type)
+{
+	struct phonebook_field *url;
+
+	if (url_val == NULL || strlen(url_val) == 0)
+		return;
+
+	/* Not adding url if there is already added with the same value */
+	if (find_field(contact->urls, url_val, type))
+		return;
+
+	url = g_new0(struct phonebook_field, 1);
+
+	url->text = g_strdup(url_val);
+	url->type = type;
+
+	contact->urls = g_slist_append(contact->urls, url);
+}
+
 static GString *gen_vcards(GSList *contacts,
 					const struct apparam_field *params)
 {
@@ -1331,7 +1378,6 @@ static void contact_init(struct phonebook_contact *contact, char **reply)
 	contact->suffix = g_strdup(reply[COL_NAME_SUFFIX]);
 	contact->birthday = g_strdup(reply[COL_BIRTH_DATE]);
 	contact->nickname = g_strdup(reply[COL_NICKNAME]);
-	contact->website = g_strdup(reply[COL_URL]);
 	contact->photo = g_strdup(reply[COL_PHOTO]);
 	contact->company = g_strdup(reply[COL_ORG_NAME]);
 	contact->department = g_strdup(reply[COL_ORG_DEPARTMENT]);
@@ -1407,17 +1453,18 @@ static void contact_add_numbers(struct phonebook_contact *contact,
 	g_strfreev(aff_numbers);
 }
 
-static enum phonebook_email_type get_email_type(const char *affilation)
+static enum phonebook_field_type get_field_type(const char *affilation)
 {
 	if (g_strcmp0(AFFILATION_HOME, affilation) == 0)
-		return EMAIL_TYPE_HOME;
+		return FIELD_TYPE_HOME;
 	else if (g_strcmp0(AFFILATION_WORK, affilation) == 0)
-		return EMAIL_TYPE_WORK;
+		return FIELD_TYPE_WORK;
 
-	return EMAIL_TYPE_OTHER;
+	return FIELD_TYPE_OTHER;
 }
 
-static void add_aff_email(struct phonebook_contact *contact, char *aff_email)
+static void add_aff_field(struct phonebook_contact *contact, char *aff_email,
+						add_field_t add_field_cb)
 {
 	char **email_parts;
 	char *type, *email;
@@ -1440,7 +1487,7 @@ static void add_aff_email(struct phonebook_contact *contact, char *aff_email)
 	else
 		goto failed;
 
-	add_email(contact, email, get_email_type(type));
+	add_field_cb(contact, email, get_field_type(type));
 
 failed:
 	g_strfreev(email_parts);
@@ -1457,50 +1504,11 @@ static void contact_add_emails(struct phonebook_contact *contact,
 
 	if (aff_emails)
 		for(i = 0; aff_emails[i] != NULL; ++i)
-			add_aff_email(contact, aff_emails[i]);
+			add_aff_field(contact, aff_emails[i], add_email);
 
 	g_strfreev(aff_emails);
 }
 
-static enum phonebook_address_type get_addr_type(const char *affilation)
-{
-	if (g_strcmp0(AFFILATION_HOME, affilation) == 0)
-		return ADDR_TYPE_HOME;
-	else if (g_strcmp0(AFFILATION_WORK, affilation) == 0)
-		return ADDR_TYPE_WORK;
-
-	return ADDR_TYPE_OTHER;
-}
-
-static void add_aff_address(struct phonebook_contact *contact, char *aff_addr)
-{
-	char **addr_parts;
-	char *type, *address;
-
-	/* Addresses from affilation data, are represented as real address
-	 * string and affilation type - those strings are separated by
-	 * SUB_DELIM string */
-	addr_parts = g_strsplit(aff_addr, SUB_DELIM, 2);
-
-	if (!addr_parts)
-		return;
-
-	if (addr_parts[0])
-		address = addr_parts[0];
-	else
-		goto failed;
-
-	if (addr_parts[1])
-		type = addr_parts[1];
-	else
-		goto failed;
-
-	add_address(contact, address, get_addr_type(type));
-
-failed:
-	g_strfreev(addr_parts);
-}
-
 static void contact_add_addresses(struct phonebook_contact *contact,
 								char **reply)
 {
@@ -1513,11 +1521,26 @@ static void contact_add_addresses(struct phonebook_contact *contact,
 
 	if (aff_addr)
 		for(i = 0; aff_addr[i] != NULL; ++i)
-			add_aff_address(contact, aff_addr[i]);
+			add_aff_field(contact, aff_addr[i], add_address);
 
 	g_strfreev(aff_addr);
 }
 
+static void contact_add_urls(struct phonebook_contact *contact, char **reply)
+{
+	char **aff_url;
+	int i;
+
+	/* Addresses from affilation */
+	aff_url = g_strsplit(reply[COL_URL], MAIN_DELIM, MAX_FIELDS);
+
+	if (aff_url)
+		for(i = 0; aff_url[i] != NULL; ++i)
+			add_aff_field(contact, aff_url[i], add_url);
+
+	g_strfreev(aff_url);
+}
+
 static void contact_add_organization(struct phonebook_contact *contact,
 								char **reply)
 {
@@ -1595,6 +1618,7 @@ add_numbers:
 	contact_add_numbers(contact, reply);
 	contact_add_emails(contact, reply);
 	contact_add_addresses(contact, reply);
+	contact_add_urls(contact, reply);
 	contact_add_organization(contact, reply);
 
 	DBG("contact %p", contact);
@@ -1903,7 +1927,7 @@ void *phonebook_get_entry(const char *folder, const char *id,
 	if (strncmp(id, CONTACT_ID_PREFIX, strlen(CONTACT_ID_PREFIX)) == 0)
 		query = g_strdup_printf(CONTACTS_QUERY_FROM_URI, id, id, id, id,
 						id, id, id, id, id, id, id, id,
-						id, id, id);
+						id, id);
 	else
 		query = g_strdup_printf(CONTACTS_OTHER_QUERY_FROM_URI,
 								id, id, id);
